package com.manager.sport.ua.web.api;

import static org.springframework.util.Assert.hasText;
import static org.springframework.util.Assert.isNull;
import static org.springframework.util.Assert.notNull;

import java.util.Collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.manager.sport.ua.db.AthleteRepository;
import com.manager.sport.ua.db.CoachRepository;
import com.manager.sport.ua.entity.Athlete;
import com.manager.sport.ua.entity.Coach;
import com.manager.sport.ua.entity.Planification;
import com.manager.sport.ua.entity.mappings.Mapping;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@RestController
@RequestMapping(value="/v1/athletes")
public class AthleteController {
	
	private final AthleteRepository athleteRepository;
	private final CoachRepository coachRepository;
	private final MongoTemplate mongoTemplate;

	@Autowired
	public AthleteController(
			AthleteRepository repository, 
			CoachRepository coachRepository,
			MongoTemplate mongoTemplate) {
		this.athleteRepository = repository;
		this.coachRepository = coachRepository;
		this.mongoTemplate = mongoTemplate;
	}
	
	@RequestMapping(method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE, 
			produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@Transactional
	public ResponseEntity<Athlete> createAthlete(@RequestBody Athlete athlete){
		isNull(athleteRepository.findByEmail(athlete.getEmail()), "Athlete email must not exists"); //TODO - Validations
		mongoTemplate.updateFirst(
	            Query.query(Criteria.where("email").is(athlete.getEmailCoach())), 
	            new Update().push("athletes", Mapping.athleteToAthleteSimple(athlete)), Coach.class);
		return new ResponseEntity<>(athleteRepository.save(athlete), HttpStatus.OK);
	}
	
	@RequestMapping(method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE, 
			produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@Transactional
	public ResponseEntity<Athlete> updateAthlete(@RequestBody Athlete athlete){
		notNull(athleteRepository.findByEmail(athlete.getEmail()), "Athlete email must exists"); //TODO - Validations
		
		return new ResponseEntity<>(athleteRepository.save(athlete), HttpStatus.OK);
	}
	
	@RequestMapping(method = RequestMethod.GET, value="/{email}/", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@Transactional(readOnly=true)
	public ResponseEntity<Athlete> getAthleteInfo(@PathVariable("email") String email){
		hasText(email, "Email of the athlete must not be null");
		notNull(athleteRepository.findByEmail(email), "Athlete email doesn't exists");

		return new ResponseEntity<>(athleteRepository.findByEmail(email), HttpStatus.OK);
	}
	
	@RequestMapping(method = RequestMethod.GET, value="/{coach}/", consumes = MediaType.APPLICATION_JSON_UTF8_VALUE, 
			produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@Transactional(readOnly=true)
	public ResponseEntity<Collection<Athlete>> getAthletes(@PathVariable("coach") String emailCoach){
		notNull(emailCoach, "Email of the coach must not be null");
		notNull(coachRepository.findByEmail(emailCoach), "Email Coach doesn't exists");
		Coach coach = coachRepository.findOne(emailCoach);
		log.info("" + coach);
		return new ResponseEntity<>(athleteRepository.findByEmailCoach(emailCoach), HttpStatus.OK);
	}
	
	@RequestMapping(method = RequestMethod.PUT, value="/{email}/", consumes = MediaType.APPLICATION_JSON_UTF8_VALUE, 
			produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@Transactional(readOnly=true)
	public ResponseEntity<Athlete> pushPlanification(
			@PathVariable("email") String email,
			@RequestBody Planification planification){
		notNull(email, "Email must not be null");
		notNull(athleteRepository.findByEmail(email), "Email athlete doesn't exists");
		
		mongoTemplate.updateFirst(
	            Query.query(Criteria.where("email").is(email)), 
	            new Update().set("planification", planification), Athlete.class);
		
		return new ResponseEntity<>(athleteRepository.findByEmail(email), HttpStatus.OK);
	}

}
