package com.manager.sport.ua.web.api;

import static org.springframework.util.Assert.isNull;
import static org.springframework.util.Assert.notNull;

import java.time.LocalDate;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.manager.sport.ua.db.CoachRepository;
import com.manager.sport.ua.db.UserRepository;
import com.manager.sport.ua.entity.Coach;
import com.manager.sport.ua.entity.mappings.Mapping;


@RestController
@RequestMapping(value="/v1/coaches")
public class CoachController {
	
	private final CoachRepository coachRepository;
	private final UserRepository userRepository;

	@Autowired
	public CoachController(
			CoachRepository repository,
			UserRepository userRepository) {
		this.coachRepository = repository;
		this.userRepository = userRepository;
	}
	
	
	@RequestMapping(method = RequestMethod.GET, value="/{email}/", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@Transactional(readOnly=true)
	public ResponseEntity<Coach> getCoach(@PathVariable String email){
		Coach coachFound = coachRepository.findOne(email);
		return new ResponseEntity<>(coachFound, HttpStatus.OK);
	}
	
	@RequestMapping(method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE, 
			produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@Transactional
	public ResponseEntity<Coach> newCoach(@RequestBody Coach user){
		isNull(coachRepository.findOne(user.getEmail()), "Email already exists"); //TODO - Validations
		user.setDateRegister(LocalDate.now());
		userRepository.save(Mapping.coachToUser(user));
		coachRepository.save(user);
		
		Coach coachInserted = coachRepository.findOne(user.getEmail());
		
		return new ResponseEntity<>(coachInserted, HttpStatus.OK);
	}
	
	@RequestMapping(method = RequestMethod.PUT, consumes = MediaType.APPLICATION_JSON_UTF8_VALUE, 
			produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@Transactional(readOnly=true)
	public ResponseEntity<Coach> updateCoach(@RequestBody Coach user){
		notNull(coachRepository.findByEmail(user.getEmail()), "Email doesn't exists");
		
		return new ResponseEntity<>(coachRepository.save(user), HttpStatus.OK);
	}
	
}
