package com.manager.sport.ua.web.view;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import com.manager.sport.ua.entity.Athlete;
import com.manager.sport.ua.entity.Planification;
import com.manager.sport.ua.entity.Workout;
import com.manager.sport.ua.entity.WorkoutEndurance;
import com.manager.sport.ua.entity.WorkoutStrength;
import com.manager.sport.ua.web.api.AthleteController;
import com.manager.sport.ua.web.api.CoachController;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Controller
public class CoachViewController {

	private final CoachController coachController;
	private final AthleteController athleteController;

	@Autowired
	public CoachViewController(CoachController coachController, AthleteController athleteController) {
		super();
		this.coachController = coachController;
		this.athleteController = athleteController;
	}

	@RequestMapping("/")
	String index() {
		return "index";
	}

	@RequestMapping(value = "/coach/athletes/{emailCoach}/", method = RequestMethod.GET)
	String athletes(@PathVariable("emailCoach") String emailCoach, Model model) {

		// ----------MOCK--------------
		List<Athlete> athletes =  (List<Athlete>) athleteController.getAthletes(emailCoach).getBody();
		model.addAttribute("coachEmail", emailCoach);
		model.addAttribute("athletes", athletes);// coachController.getCoach(email));

		return "athletes";
	}

	@RequestMapping("/coach/newAthlete/{emailCoach}/")
	String newAthlete(@PathVariable("emailCoach") String emailCoach, @ModelAttribute Athlete athlete,  BindingResult errors, Model model) {

		model.addAttribute("coachEmail", emailCoach);

		return "newAthlete";
	}
	
	@RequestMapping("/coach/athlete/{athleteEmail}/")
	String athletePlanification(@PathVariable("athleteEmail") String athleteEmail, @RequestParam(required= false) String date,  Model model) {

		ResponseEntity<Athlete> response = athleteController.getAthleteInfo(athleteEmail);
		Athlete athlete = response.getBody();
		String fecha;
		
		if(null == date){
			fecha = LocalDate.now().toString();
		} else{
			fecha = date;
		}
		
		model.addAttribute("urlPut", "/v1/athletes/" + athleteEmail + "/");
		model.addAttribute("urlGet", "/coach/athlete" + athleteEmail + "/?date=");
		model.addAttribute("athleteEmail", athleteEmail);
		model.addAttribute("coachEmail", athlete.getEmailCoach());
		if(athlete.getPlanification().getWorkouts() != null && !athlete.getPlanification().getWorkouts().isEmpty()){
			Optional<Workout> workout = athlete.getPlanification().getWorkouts()
					.stream()
					.filter(work -> (work.getDateWorkout().equals(fecha)))
					.findFirst();
			
			if(workout.isPresent())
			{
				model.addAttribute("workout", workout.get());
				
				List<WorkoutEndurance> endurances = workout.get().getWorkoutsEndurance();
				List<WorkoutStrength> strength = workout.get().getWorkoutsStrength();
				
				model.addAttribute("endurances", endurances);
				model.addAttribute("strengths", strength);
				
			} else {
				Workout workVoidInit = new Workout();
				workVoidInit.setComments("");
				workVoidInit.setDataExternalLoad("");
				workVoidInit.setWorkoutsEndurance(new ArrayList<>());
				workVoidInit.setWorkoutsStrength(new ArrayList<>());
				workVoidInit.setDateWorkout(fecha);
				model.addAttribute("workout", workVoidInit);
			}
		} else {
			Workout workVoidInit = new Workout();
			workVoidInit.setComments("");
			workVoidInit.setDataExternalLoad("");
			workVoidInit.setWorkoutsEndurance(new ArrayList<>());
			workVoidInit.setWorkoutsStrength(new ArrayList<>());
			workVoidInit.setDateWorkout(fecha);
			model.addAttribute("workout", workVoidInit);
		}
		

		return "athletePlanification";
	}
	
	
	@RequestMapping(value = "/coach/createAthlete/{emailCoach}/", method = RequestMethod.POST)
	String createUser(@ModelAttribute Athlete athlete, @PathVariable String emailCoach, BindingResult errors, Model model) {
		log.info("Creating athlete...");
		
		//Init planification
		Planification planification = new Planification();
		List<Workout> workouts = new ArrayList<>();
		planification.setWorkouts(workouts);
		athlete.setPlanification(planification);
		//--
		
		athlete.setEmailCoach(emailCoach);
		
		if(athleteController.createAthlete(athlete).getStatusCode().equals(HttpStatus.OK))
		{
			return "redirect:/coach/athlete/" + athlete.getEmail() + "/";
		}
		return "redirect:errorLogin";
	 }

}
