package com.manager.sport.ua.web.view;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import com.manager.sport.ua.entity.Coach;
import com.manager.sport.ua.entity.User;
import com.manager.sport.ua.web.api.CoachController;
import com.manager.sport.ua.web.api.UserController;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Controller
public class UserViewController {

	private final UserController userController;
	private final CoachController coachController;

	@Autowired
	public UserViewController(UserController userController, CoachController coachController) {
		super();
		this.userController = userController;
		this.coachController = coachController;
	}

	//SIGNUP
	
	@RequestMapping("/signup")
	String signup(@ModelAttribute Coach coach, BindingResult errors, Model model) {
		return "signup";
	}

	
	@RequestMapping(value = "/createUser", method = RequestMethod.POST)
	String createUser(@ModelAttribute Coach coach, BindingResult errors, Model model) {
		log.info("Creating user...");
		if (coachController.newCoach(coach).getStatusCode().equals(HttpStatus.OK)) {
			return "redirect:coach/athletes/" + coach.getEmail() + "/";
		}
		return "redirect:signup";
	}
	
	@RequestMapping("/errorLogin")
	String error() {
		return "errorLogin";
	}

	//LOGIN
	
	@RequestMapping("/login")
	String login(@ModelAttribute User user, BindingResult errors, Model model) {
		return "login";
	}
	
	@RequestMapping(value = "/loginUser", method = RequestMethod.POST)
	String User(@ModelAttribute User user, BindingResult errors, Model model) {
		boolean isAuthenticated = userController.autenticate(user).getBody().booleanValue();
		String userParam = user.getEmail();
		if(isAuthenticated)
			return "redirect:coach/athletes/" + userParam + "/"; 
		return "redirect:login";
	}

	

	// Login form with error
	@RequestMapping("/login-error.html")
	public String loginError(Model model) {
		model.addAttribute("loginError", true);
		return "login.html";
	}

	@RequestMapping("/hello")
	String hello() {
		return "hello";
	}

}
